/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie;

import java.lang.reflect.Method;

import javax.persistence.EntityManager;

/**
 * Support class to allow using Fonzie instrumented classes in unit tests. Will require either :
 * <ul>
 * <li>aspectJ compiler to instrument test classes</li>
 * <li>aspectJ weaver set as javaagent for load-time-weaving</li>
 * </ul>
 *
 * @author <a href="nicolas@apache.org">Nicolas De Loof</a>
 */
public class ORMAspectTestSupport
{
    public static void setupAspect( EntityManager entityManager )
        throws Exception
    {
        setupAspect( entityManager, new DefaultQueryExecutor( entityManager ) );
    }

    public static void setupAspect( EntityManager entityManager, QueryExecutor queryExecutor )
        throws Exception
    {

        AbstractORMAspect aspect = aspectOf( ORMAspect.class );
        aspect.entityManager = entityManager;
        aspect.setQueryExecutor( queryExecutor );
    }

    private static <T> T aspectOf( Class<T> clazz )
        throws Exception
    {
        Method aspectOf = clazz.getMethod( "aspectOf", (Class[]) null );
        return (T) aspectOf.invoke( clazz, (Object[]) null );
    }
}
