/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.parsers;

import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.loof.fonzie.PersistenceQuery;
import fr.loof.fonzie.QueryParser;
import fr.loof.fonzie.keywords.Aggregate;

/**
 * @author <a href="nicolas@apache.org">Nicolas De Loof</a>
 */
public class CompoundQueryParser extends AbstractQueryParser
    implements QueryParser
{
    private static Logger logger = LoggerFactory.getLogger( JPQLPersistenceQuery.class );

    public List<QueryParser> parsers = new LinkedList<QueryParser>();

    public CompoundQueryParser()
    {
        parsers.add( new NamedQueryParser() );
        parsers.add( new KeywordParser() );
    }

    protected PersistenceQuery doParse( Class entityClass, Aggregate aggregate, String criteria, Class[] types )
    {
        for ( QueryParser parser : parsers )
        {
            PersistenceQuery query = parser.parse( entityClass, aggregate, criteria, types );
            if ( query != null )
            {
                logger.debug( "query succesfully parsed by {}", parser.getClass() );
                return query;
            }
        }
        throw new IllegalStateException( "Failed to parse persistence method " + entityClass + "#" + criteria );
    }
}
